@@ -34,7 +34,10 @@
 
 (defun riece-handle-302-message (prefix number name string)
   "RPL_USERHOST \":*1<reply> *( \" \" <reply> )\""
-  (let ((replies (split-string (substring string 1) " ")))
+  (let ((replies (split-string (if (eq (aref string 0) ?:)
+				   (substring string 1)
+				 string)
+			       " ")))
     (while replies
       (if (string-match
 	   (concat "^\\(" riece-user-regexp
@@ -82,12 +85,15 @@
 		(riece-format-identity
 		 (riece-make-identity user riece-server-name)
 		 t))
-	      (split-string (substring string 1) " ")
+	      (split-string (if (eq (aref string 0) ?:)
+				(substring string 1)
+			      string)
+			    " ")
 	      "")))
     "\n")))
 
 (defun riece-handle-301-message (prefix number name string)
-  (if (string-match (concat "^\\(" riece-user-regexp "\\) :") string)
+  (if (string-match (concat "^\\(" riece-user-regexp "\\) :?") string)
       (let ((user (match-string 1 string))
 	    (message (substring string (match-end 0))))
 	(riece-user-toggle-away user t)
@@ -122,7 +128,7 @@
 (defun riece-handle-311-message (prefix number name string)
   (if (string-match
        (concat "^\\(" riece-user-regexp
-	       "\\) \\([^ ]+\\) \\([^ ]+\\) \\* :")
+	       "\\) \\([^ ]+\\) \\([^ ]+\\) \\* :?")
        string)
       (let ((user (match-string 1 string))
 	    (name (substring string (match-end 0)))
@@ -142,7 +148,7 @@
 
 (defun riece-handle-312-message (prefix number name string)
   (if (string-match
-       (concat "^\\(" riece-user-regexp "\\) \\([^ ]+\\) :")
+       (concat "^\\(" riece-user-regexp "\\) \\([^ ]+\\) :?")
        string)
       (riece-insert-info
        (list riece-dialogue-buffer riece-others-buffer)
@@ -168,7 +174,7 @@
 
 (defun riece-handle-317-message (prefix number name string)
   (if (string-match
-       (concat "^\\(" riece-user-regexp "\\) \\([0-9]+\\) [^:]*:seconds")
+       (concat "^\\(" riece-user-regexp "\\) \\([0-9]+\\) ")
        string)
       (let ((user (match-string 1 string))
 	    (idle (match-string 2 string)))
@@ -184,7 +190,7 @@
 	  "\n")))))
 
 (defun riece-handle-319-message (prefix number name string)
-  (if (string-match (concat "^\\(" riece-user-regexp "\\) :") string)
+  (if (string-match (concat "^\\(" riece-user-regexp "\\) :?") string)
       (let ((user (match-string 1 string))
 	    (channels
 	     (mapconcat
@@ -212,7 +218,7 @@
 	  "\n")))))
 
 (defun riece-handle-351-message (prefix number name string)
-  (if (string-match "\\([^ ]+\\.[^ ]+\\) \\([^ ]+\\) :" string)
+  (if (string-match "\\([^ ]+\\.[^ ]+\\) \\([^ ]+\\) :?" string)
       (riece-insert-info
        (list riece-dialogue-buffer riece-others-buffer)
        (concat
@@ -223,48 +229,23 @@
 		 (substring string (match-end 0))))
 	"\n"))))
 
-(defvar riece-353-users nil)
+(defvar riece-353-message-alist nil)
 (defun riece-handle-353-message (prefix number name string)
   "RPL_NAMREPLY	\"[=\*@] <channel> :[[@|+]<nick> [[@|+]<nick> [...]]]\"."
-  (if (string-match "^[=\*@] *\\([^ ]+\\) +:" string)
-      (let ((channel (match-string 1 string))
-	    (start 0)
-	    user)
-	(make-local-variable 'riece-353-users)
-	(setq string (substring string (match-end 0)))
-	(while (string-match
-		(concat "\\([@+]\\)?\\(" riece-user-regexp "\\) *")
-		string start)
-	  (put-text-property (match-beginning 2) (match-end 2)
-			     'riece-identity
-			     (riece-make-identity (match-string 2 string)
-						  riece-server-name)
-			     string)
-	  (setq start (match-end 0)
-		user (if (match-beginning 1)
-			 (if (eq (aref string (match-beginning 1)) ?@)
-			     (list (match-string 2 string) ?o)
-			   (if (eq (aref string (match-beginning 1)) ?+)
-			       (list (match-string 2 string) ?v)))
-		       (list (match-string 2 string)))
-		riece-353-users (cons user riece-353-users)))
-	(let* ((channel-identity (riece-make-identity channel
-						      riece-server-name))
-	       (buffer (riece-channel-buffer channel-identity)))
-	  (riece-insert-info buffer (concat "Users: " string "\n"))
-	  (riece-insert-info
-	   (if (and riece-channel-buffer-mode
-		    (not (eq buffer riece-channel-buffer)))
-	       (list riece-dialogue-buffer riece-others-buffer)
-	     riece-dialogue-buffer)
-	   (concat
-	    (riece-concat-server-name
-	     (format "Users on %s: %s"
-		     (riece-format-identity channel-identity t) string))
-	    "\n"))))))
+  (make-local-variable 'riece-353-message-alist)      
+  (if (string-match "^[=\*@] *\\([^ ]+\\) +:?" string)
+      (let* ((channel (match-string 1 string))
+	     (entry (assoc channel riece-353-message-alist)))
+	(if entry
+	    (setcdr entry
+		    (concat (cdr entry)
+			    (substring string (match-end 0))))
+	  (setq riece-353-message-alist
+		(cons (cons channel (substring string (match-end 0)))
+		      riece-353-message-alist))))))
 
 (defun riece-handle-322-message (prefix number name string)
-  (if (string-match "^\\([^ ]+\\) \\([0-9]+\\) :" string)
+  (if (string-match "^\\([^ ]+\\) \\([0-9]+\\) :?" string)
       (let* ((channel (match-string 1 string))
 	     (visible (match-string 2 string))
 	     (topic (substring string (match-end 0))))
@@ -308,7 +289,7 @@
 	    "\n"))))))
 
 (defun riece-handle-set-topic (prefix number name string remove)
-  (if (string-match "^\\([^ ]+\\) :" string)
+  (if (string-match "^\\([^ ]+\\) :?" string)
       (let* ((channel (match-string 1 string))
 	     (message (substring string (match-end 0)))
 	     (channel-identity (riece-make-identity channel riece-server-name))
@@ -428,11 +409,51 @@
 
 (defun riece-handle-366-message (prefix number name string)
   "RPL_ENDOFNAMES \"<channel> :End of NAMES list\""
-  (if (string-match "^\\([^ ]+\\) +:" string)
-      (let ((channel (match-string 1 string)))
-	(riece-naming-assert-channel-users (nreverse riece-353-users)
-					   channel)))
-  (setq riece-353-users nil))
+  (if (string-match "^\\([^ ]+\\) " string)
+      (let* ((channel (match-string 1 string))
+	     (channel-identity (riece-make-identity channel
+						    riece-server-name))
+	     (buffer (riece-channel-buffer channel-identity))
+	     (entry (assoc channel riece-353-message-alist))
+	     (string (cdr entry))
+	     (start 0)
+	     users)
+	(if entry
+	    (setq riece-353-message-alist
+		  (delq entry riece-353-message-alist)))
+	(while (string-match
+		(concat "\\([@+]\\)?\\(" riece-user-regexp "\\) *")
+		string start)
+	  (put-text-property (match-beginning 2) (match-end 2)
+			     'riece-identity
+			     (riece-make-identity (match-string 2 string)
+						  riece-server-name)
+			     string)
+	  (setq start (match-end 0)
+		users (cons (if (match-beginning 1)
+				(if (eq (aref string (match-beginning 1)) ?@)
+				    (list (match-string 2 string) ?o)
+				  (if (eq (aref string (match-beginning 1)) ?+)
+				      (list (match-string 2 string) ?v)))
+			      (list (match-string 2 string)))
+			    users)))
+	(setq users (nreverse users))
+	(riece-naming-assert-channel-users users channel)
+	(riece-insert-info
+	 buffer
+	 (concat (format "%d users: " (length users)) string "\n"))
+	(riece-insert-info
+	 (if (and riece-channel-buffer-mode
+		  (not (eq buffer riece-channel-buffer)))
+	     (list riece-dialogue-buffer riece-others-buffer)
+	   riece-dialogue-buffer)
+	 (concat
+	  (riece-concat-server-name
+	   (concat (format "%d users on %s: "
+			   (length users)
+			   (riece-format-identity channel-identity t))
+		   string))
+	  "\n")))))
 
 (provide 'riece-300)
 
