@@ -144,7 +144,7 @@
       (setq addons (cdr addons)))
     dependencies))
 
-(defun riece-resolve-addons (addons)
+(defun riece-resolve-addon-dependencies (addons)
   (let ((pointer addons)
 	dependencies queue)
     ;; Uniquify, first.
@@ -179,6 +179,14 @@
 	(error "Circular add-on dependency found"))
     (nreverse addons)))
 
+(defun riece-resolve-addons (addons)
+  (riece-resolve-addon-dependencies
+   (append addons
+	   (mapcar
+	    (lambda (name)
+	      (intern (file-name-sans-extension name)))
+	    (directory-files riece-addon-directory nil "\\`[^.]" t)))))
+
 (defun riece-insinuate-addon (addon &optional verbose)
   (require addon)		;implicit dependency
   (funcall (intern (concat (symbol-name addon) "-insinuate")))
@@ -219,7 +227,7 @@ All normal editing commands are turned off."
   (kill-all-local-variables)
   (buffer-disable-undo)
   (setq major-mode 'riece-addon-list-mode
-        mode-name "AddOns"
+	mode-name "AddOns"
 	mode-line-buffer-identification
 	(riece-mode-line-buffer-identification '("Riece: %12b"))
 	truncate-lines t
@@ -288,18 +296,18 @@ Useful keys:
    (list
     (or (if (eq major-mode 'riece-addon-list-mode)
 	    (get-text-property (point) 'riece-addon))
-	(completing-read "Add-on: "
-			 (mapcar (lambda (addon)
-				   (list (symbol-name addon)))
-				 riece-addons)
-			 (lambda (pointer)
-			   (let ((enabled
-				  (intern-soft (concat (symbol-name
-							(car pointer))
-						       "-enabled"))))
-			     (and enabled
-				  (null (symbol-value enabled)))))
-			 t))))
+	(intern-soft
+	 (completing-read "Add-on: "
+			  (mapcar (lambda (addon)
+				    (list (symbol-name addon)))
+				  riece-addons)
+			  (lambda (pointer)
+			    (let ((enabled
+				   (intern-soft (concat (car pointer)
+							"-enabled"))))
+			      (and enabled
+				   (null (symbol-value enabled)))))
+			  t)))))
   (riece-enable-addon addon t)
   (let ((enabled (intern-soft (concat (symbol-name addon) "-enabled"))))
     (if (and (eq major-mode 'riece-addon-list-mode)
@@ -319,18 +327,18 @@ Useful keys:
    (list
     (or (if (eq major-mode 'riece-addon-list-mode)
 	    (get-text-property (point) 'riece-addon))
-	(completing-read "Add-on: "
-			 (mapcar (lambda (addon)
-				   (list (symbol-name addon)))
-				 riece-addons)
-			 (lambda (pointer)
-			   (let ((enabled
-				  (intern-soft (concat (symbol-name
-							(car pointer))
-						       "-enabled"))))
-			     (and enabled
-				  (symbol-value enabled))))
-			 t))))
+	(intern-soft
+	 (completing-read "Add-on: "
+			  (mapcar (lambda (addon)
+				    (list (symbol-name addon)))
+				  riece-addons)
+			  (lambda (pointer)
+			    (let ((enabled
+				   (intern-soft (concat (car pointer)
+							"-enabled"))))
+			      (and enabled
+				   (symbol-value enabled))))
+			  t)))))
   (riece-disable-addon addon t)
   (let ((enabled (intern-soft (concat (symbol-name addon) "-enabled"))))
     (if (and (eq major-mode 'riece-addon-list-mode)
@@ -344,7 +352,7 @@ Useful keys:
 	    (delete-char 1)
 	    (insert "-")
 	    (put-text-property point (point) 'riece-addon addon))))))
-    
+
 (provide 'riece-addon)
 
 ;;; riece-addon.el ends here
