@@ -112,8 +112,23 @@
 
 (defun riece-command-configure-windows ()
   (interactive)
+  "Reconfigure windows with the current layout."
   (riece-redisplay-buffers t))
 
+(defun riece-command-suspend-resume ()
+  (interactive)
+  "Save or restore the current window configuration."
+  (let ((entry (assq 'riece-window-configuration (frame-parameters))))
+    (modify-frame-parameters (selected-frame)
+			     (list (cons 'riece-window-configuration
+					 (current-window-configuration))))
+    (if entry
+	(set-window-configuration (cdr entry))
+      (delete-other-windows))
+    (message
+     (substitute-command-keys
+      "\\[riece-command-suspend-resume] to get back the last windows"))))
+
 (defun riece-command-change-layout (name)
   "Select a layout-name from all current available layouts and change
 the layout to the selected layout-name."
@@ -435,32 +450,25 @@ the layout to the selected layout-name."
       (riece-join-channel target)
       (riece-switch-to-channel target))))
 
-(defun riece-command-join (target &optional key)
+(defun riece-command-join (target)
   (interactive
-   (let* ((completion-ignore-case t)
-	  (target
-	   (if riece-join-channel-candidate
-	       (let ((default (riece-format-identity
-			       riece-join-channel-candidate)))
-		 (riece-completing-read-identity
-		  (format "Join channel/user (default %s): " default)
-		  (riece-get-identities-on-server (riece-current-server-name))
-		  nil nil nil nil default))
-	     (riece-completing-read-identity
-	      "Join channel/user: "
-	      (riece-get-identities-on-server (riece-current-server-name)))))
-	  key)
-     (if (and current-prefix-arg
-	      (riece-channel-p (riece-identity-prefix target)))
-	 (setq key
-	       (riece-read-passwd (format "Key for %s: "
-					  (riece-format-identity target)))))
-     (list target key)))
+   (let ((completion-ignore-case t))
+     (list
+      (if riece-join-channel-candidate
+	  (let ((default (riece-format-identity
+			  riece-join-channel-candidate)))
+	    (riece-completing-read-identity
+	     (format "Join channel/user (default %s): " default)
+	     (riece-get-identities-on-server (riece-current-server-name))
+	     nil nil nil nil default))
+	(riece-completing-read-identity
+	 "Join channel/user: "
+	 (riece-get-identities-on-server (riece-current-server-name)))))))
   (let ((pointer (riece-identity-member target riece-current-channels)))
     (if pointer
 	(riece-command-switch-to-channel (car pointer))
       (if (riece-channel-p (riece-identity-prefix target))
-	  (riece-command-join-channel target key)
+	  (riece-command-join-channel target nil)
 	(riece-command-join-partner target)))))
 
 (defun riece-command-part-channel (target message)
@@ -473,7 +481,7 @@ the layout to the selected layout-name."
 				 (format "PART %s\r\n"
 					 (riece-identity-prefix target))))))
 
-(defun riece-command-part (target &optional message)
+(defun riece-command-part (target)
   (interactive
    (progn
      (riece-check-channel-commands-are-usable)
@@ -484,10 +492,10 @@ the layout to the selected layout-name."
 		      (riece-format-identity riece-current-channel))
 	      riece-current-channels nil nil nil nil
 	      (riece-format-identity riece-current-channel)))
-	    message)
-       (if (and current-prefix-arg
-		(riece-channel-p (riece-identity-prefix target)))
-	   (setq message (read-string "Message: ")))
+	    (message
+	     (if current-prefix-arg
+		 (read-string "Message: ")
+	       riece-part-message)))
        (list target message))))
   (if (riece-identity-member target riece-current-channels)
       (if (riece-channel-p (riece-identity-prefix target))
@@ -616,7 +624,7 @@ If prefix argument ARG is non-nil, toggle frozen status."
 	(let ((message
 	       (if arg
 		   (read-string "Message: ")
-	       riece-quit-message))
+		 riece-quit-message))
 	      (alist riece-server-process-alist))
 	  (while alist
 	    (riece-quit-server-process (cdr (car alist)) message)
@@ -680,6 +688,7 @@ If prefix argument ARG is non-nil, toggle frozen status."
 	(if (equal current completion)
 	    (with-output-to-temp-buffer "*Help*"
 	      (display-completion-list all))
+	  (re-search-forward "\\>" nil t)
 	  (delete-region (point) (- (point) (length current)))
 	  (insert completion))))))
 
