@@ -174,7 +174,7 @@ the `riece-server-keyword-map' variable."
       (unless (riece-queue-empty riece-send-queue)
 	(riece-run-at-time riece-send-delay nil
 			   (lambda (process)
-			     (if (process-live-p process)
+			     (if (riece-server-process-opened process)
 				 (riece-flush-send-queue process)))
 			   process)))))
 
@@ -266,6 +266,8 @@ the `riece-server-keyword-map' variable."
     (erase-buffer)))
 
 (defun riece-close-server-process (process)
+  (with-current-buffer (process-buffer process)
+    (run-hooks 'riece-after-close-hook))
   (kill-buffer (process-buffer process))
   (setq riece-server-process-alist
 	(delq (rassq process riece-server-process-alist)
@@ -286,6 +288,15 @@ the `riece-server-keyword-map' variable."
 	      (throw 'found t))
 	  (setq alist (cdr alist)))))))
 
+(defun riece-server-properties (server-name)
+  "Return a list of properties associated with SERVER-NAME."
+  (if (equal server-name "")
+      riece-server
+    (let ((entry (assoc server-name riece-server-alist)))
+      (unless entry
+	(error "No such server"))
+      (cdr entry))))
+
 (provide 'riece-server)
 
 ;;; riece-server.el ends here
