@@ -58,7 +58,8 @@
     (riece-dialogue-buffer "*Dialogue*" riece-dialogue-mode)
     (riece-others-buffer "*Others*" riece-others-mode)
     (riece-channel-list-buffer "*Channels*" riece-channel-list-mode)
-    (riece-user-list-buffer " *Users*" riece-user-list-mode)))
+    (riece-user-list-buffer " *Users*" riece-user-list-mode)
+    (riece-temp-buffer " *Temp*")))
 
 (defvar riece-shrink-buffer-idle-timer nil
   "Timer object to periodically shrink channel buffers.")
@@ -67,7 +68,8 @@
   "Non nil if add-ons are already insinuated.")
 
 (defvar riece-select-keys
-  `("1" riece-command-switch-to-channel-by-number-1
+  `("#" riece-command-switch-to-channel-by-number
+    "1" riece-command-switch-to-channel-by-number-1
     "2" riece-command-switch-to-channel-by-number-2
     "3" riece-command-switch-to-channel-by-number-3
     "4" riece-command-switch-to-channel-by-number-4
@@ -183,7 +185,8 @@ If optional argument SAFE is nil, overwrite previous definitions."
     "r" riece-command-configure-windows
     "x" riece-command-copy-region
     "t" riece-command-topic
-    "w" riece-command-who)
+    "w" riece-command-who
+    "z" riece-command-suspend-resume)
 
   (riece-define-keys riece-command-mode-map
     "\r" riece-command-enter-message
@@ -274,6 +277,9 @@ If optional argument CONFIRM is non-nil, ask which IRC server to connect."
   (run-hooks 'riece-after-load-startup-hook)
   (if (riece-server-opened)
       (riece-command-configure-windows)
+    (modify-frame-parameters (selected-frame)
+			     (list (cons 'riece-window-configuration
+					 (current-window-configuration))))
     (unless riece-addons-insinuated
       (setq riece-addons (riece-resolve-addons riece-addons))
       (let ((pointer riece-addons))
@@ -293,11 +299,13 @@ If optional argument CONFIRM is non-nil, ask which IRC server to connect."
 	       (lambda ()
 		 (let ((buffers riece-buffer-list))
 		   (while buffers
-		     (if (eq (derived-mode-class
-			      (with-current-buffer (car buffers)
-				major-mode))
-			     'riece-dialogue-mode)
-			 (riece-shrink-buffer (car buffers)))
+		     (if (buffer-live-p (car buffers))
+			 (if (eq (derived-mode-class
+				  (with-current-buffer (car buffers)
+				    major-mode))
+				 'riece-dialogue-mode)
+			     (riece-shrink-buffer (car buffers)))
+		       (delq (car buffers) riece-buffer-list))
 		     (setq buffers (cdr buffers))))))))
     (switch-to-buffer riece-command-buffer)
     (riece-display-connect-signals)
@@ -334,13 +342,15 @@ If optional argument CONFIRM is non-nil, ask which IRC server to connect."
     (while (> (buffer-size) riece-max-buffer-size)
       (let* ((inhibit-read-only t)
 	     buffer-read-only
-	     (start (point))
-	     (end (progn (beginning-of-line 2) (point)))
-	     (overlays (riece-overlays-in start end)))
+	     (end (progn
+		    (goto-char riece-shrink-buffer-remove-chars)
+		    (beginning-of-line 2)
+		    (point)))
+	     (overlays (riece-overlays-in (point-min) end)))
 	(while overlays
 	  (riece-delete-overlay (car overlays))
 	  (setq overlays (cdr overlays)))
-	(delete-region start end)))))
+	(delete-region (point-min) end)))))
 
 (defun riece-exit ()
   (if riece-save-variables-are-dirty
@@ -363,8 +373,13 @@ If optional argument CONFIRM is non-nil, ask which IRC server to connect."
 	riece-channel-list-indicator "No channel"
 	riece-away-indicator "-"
 	riece-operator-indicator "-"
+	riece-channel-status-indicator "-"
 	riece-freeze-indicator "-")
-  (delete-other-windows)
+  (let ((window-configuration
+	 (cdr (assq 'riece-window-configuration (frame-parameters)))))
+    (if window-configuration
+	(set-window-configuration window-configuration)
+      (delete-other-windows)))
   (run-hooks 'riece-exit-hook))
 
 (defun riece-command-mode ()
@@ -384,6 +399,7 @@ For a list of the generic commands type \\[riece-command-generic] ? RET.
 
   (setq riece-away-indicator "-"
 	riece-operator-indicator "-"
+	riece-channel-status-indicator "-"
 	major-mode 'riece-command-mode
 	mode-name "Command"
 	mode-line-buffer-identification
@@ -391,6 +407,7 @@ For a list of the generic commands type \\[riece-command-generic] ? RET.
 	 '("Riece: "
 	   riece-away-indicator
 	   riece-operator-indicator
+	   riece-channel-status-indicator
 	   " "
 	   riece-user-indicator
 	   " "
@@ -426,6 +443,7 @@ Instead, these commands are available:
   (setq riece-freeze riece-default-freeze
 	riece-away-indicator "-"
 	riece-operator-indicator "-"
+	riece-channel-status-indicator "-"
 	major-mode 'riece-dialogue-mode
 	mode-name "Dialogue"
 	mode-line-buffer-identification
@@ -434,6 +452,7 @@ Instead, these commands are available:
 	   riece-away-indicator
 	   riece-operator-indicator
 	   riece-freeze-indicator
+	   riece-channel-status-indicator
 	   " "
 	   riece-channel-list-indicator " "))
 	truncate-lines nil
@@ -463,6 +482,7 @@ Instead, these commands are available:
 	   riece-away-indicator
 	   riece-operator-indicator
 	   riece-freeze-indicator
+	   riece-channel-status-indicator
 	   " "
 	   riece-long-channel-indicator))))
 
@@ -529,6 +549,97 @@ Instead, these commands are available:
 	  (funcall (nth 2 (car alist))))
 	(setq alist (cdr alist))))))
 
+(defvar reporter-prompt-for-summary-p)
+(defun riece-submit-bug-report (&optional recent-messages recent-keys)
+  "Submit via mail a bug report on Riece."
+  ;; This strange form ensures that (recent-keys) is the value before
+  ;; the bug subject string is read.
+  (interactive (list (riece-recent-messages 20) (recent-keys)))
+  (message "Querying server version...")
+  (let ((pointer riece-server-process-alist)
+	nickname)
+    (while pointer
+      (when (riece-server-process-opened (cdr (car pointer)))
+	(process-send-string (cdr (car pointer)) "VERSION\r\n")
+	(if (setq nickname
+		  (with-current-buffer (process-buffer (cdr (car pointer)))
+		    riece-real-nickname))
+	    (process-send-string
+	     (cdr (car pointer))
+	     (format "PRIVMSG %s :\1VERSION\1\r\n" nickname))))
+      (setq pointer (cdr pointer))))
+  (sit-for 3)
+  (message "Querying server version...done")
+  (require 'reporter)
+  (let ((reporter-prompt-for-summary-p t))
+    (unless riece-debug
+      (error "Please turn on riece-debug and restart Riece."))
+    (reporter-submit-bug-report
+     "liece@unixuser.org"
+     (riece-version)
+     '(riece-debug)
+     nil
+     nil
+     "This bug report will be sent to the Riece Development Team,
+not to your local site managers!!
+
+Please write in Japanese or English, because the Riece maintainers do
+not have translators to read other languages for them.
+
+Please describe as succinctly as possible:
+\t- What happened.
+\t- What you thought should have happened.
+\t- Precisely what you were doing at the time.
+
+Also include a reliable recipe for triggering the bug, as well as
+any lisp back-traces that you may have.
+\(setq stack-trace-on-error t\), or \(setq debug-on-error t\) if you
+are familiar with the debugger, to get a lisp back-trace.")
+    (delete-other-windows)
+    (save-excursion
+      (goto-char (point-max))
+      (insert
+       "\nAdd-on state:\n"
+       "------------\n"
+       (save-window-excursion
+	 (save-excursion
+	   (riece-command-list-addons)
+	   (search-forward "\n\n")
+	   (buffer-substring (point-min) (point)))))
+      (insert "Recent messages from servers:\n"
+	      "--------------------------")
+      (let ((pointer riece-server-process-alist))
+	(while pointer
+	  (insert "\n- \"" (car (car pointer)) "\", \n"
+		  (format "%S" (if (equal (car (car pointer)) "")
+				   riece-server
+				 (cdr (assoc (car (car pointer))
+					     riece-server-alist))))
+		  "\n"
+		  (if (riece-server-process-opened (cdr (car pointer)))
+		      (save-excursion
+			(set-buffer (process-buffer (cdr (car pointer))))
+			(goto-char (point-max))
+			(beginning-of-line -60)
+			(buffer-substring (point) (point-max)))
+		    "(closed server)"))
+	  (setq pointer (cdr pointer))))
+      ;; Insert recent keystrokes.
+      (insert "\n\nRecent keystrokes:\n"
+	      "-----------------\n\n")
+      (let ((before-keys (point)))
+	(insert (key-description recent-keys))
+	(save-restriction
+	  (narrow-to-region before-keys (point))
+	  (goto-char before-keys)
+	  (while (progn (move-to-column 50) (not (eobp)))
+	    (search-forward " " nil t)
+	    (insert "\n"))))
+      ;; Insert recent minibuffer messages.
+      (insert "\n\nRecent messages (most recent first):\n"
+	      "-----------------------------------\n"
+	      recent-messages))))
+
 (provide 'riece)
 
 ;;; riece.el ends here
